@@ -1,6 +1,7 @@
+from datetime import datetime
+
 import numpy as np
 import pandas as pd
-from datetime import datetime
 
 import vectorbt as vbt
 
@@ -20,6 +21,19 @@ pos_ths = [np.array([1, 1 / 2]), np.array([2, 1 / 2]), np.array([3, 1 / 2])]
 neg_ths = [np.array([1 / 2, 1 / 3]), np.array([1 / 2, 2 / 3]), np.array([1 / 2, 3 / 4])]
 
 
+# ############# Global ############# #
+
+def setup_module():
+    vbt.settings.numba['check_func_suffix'] = True
+    vbt.settings.caching.enabled = False
+    vbt.settings.caching.whitelist = []
+    vbt.settings.caching.blacklist = []
+
+
+def teardown_module():
+    vbt.settings.reset()
+
+
 # ############# generators.py ############# #
 
 class TestGenerators:
@@ -396,4 +410,4 @@ class TestGenerators:
                     (2, 'array_2', 'array_2', 'b')
                 ], names=['bolb_window', 'bolb_pos_th', 'bolb_neg_th', None])
             )
-        )
\ No newline at end of file
+        )
