@@ -1,4 +1,7 @@
-"""Ultimate Python library for time series analysis and backtesting at scale.
+# Copyright (c) 2021 Oleg Polakow. All rights reserved.
+# This code is licensed under Apache 2.0 with Commons Clause license (see LICENSE.md for details)
+
+"""Find your trading edge, using the most powerful toolkit for backtesting, algorithmic trading, and research.
 
 While there are many great backtesting packages for Python, vectorbt combines an extremely fast backtester 
 and a data science tool: it excels at processing performance and offers interactive tools to explore complex 
@@ -6,6 +9,47 @@ phenomena in trading. With it, you can traverse a huge number of strategy config
 instruments in little time, to explore where your strategy performs best and to uncover hidden patterns in data. 
 Accessing and analyzing this information for yourself could give you an information advantage in your own trading.
 
+## Installation
+
+```bash
+pip install -U vectorbt
+```
+
+To also install optional dependencies:
+
+```bash
+pip install -U "vectorbt[full]"
+```
+
+See [License](https://github.com/polakowo/vectorbt#license) notes on optional dependencies.
+
+Troubleshooting:
+
+* [TA-Lib support](https://github.com/mrjbq7/ta-lib#dependencies)
+* [Jupyter Notebook and JupyterLab support](https://plotly.com/python/getting-started/#jupyter-notebook-support)
+
+### Docker
+
+You can pull the most recent Docker image if you [have Docker installed](https://docs.docker.com/install/).
+
+```bash
+docker run --rm -p 8888:8888 -v "$PWD":/home/jovyan/work polakowo/vectorbt
+```
+
+This command pulls the latest `polakowo/vectorbt` image from Docker Hub. It then starts a container running
+a Jupyter Notebook server and exposes the server on host port 8888. Visiting `http://127.0.0.1:8888/?token=<token>`
+in a browser loads JupyterLab, where token is the secret token printed in the console. Docker destroys
+the container after notebook server exit, but any files written to the working directory in the container
+remain intact in the working directory on the host. See [Jupyter Docker Stacks - Quick Start](https://github.com/jupyter/docker-stacks#quick-start).
+
+There are two types of images:
+
+* [polakowo/vectorbt](https://hub.docker.com/r/polakowo/vectorbt): vanilla version (default)
+* [polakowo/vectorbt-full](https://hub.docker.com/r/polakowo/vectorbt-full): full version (with optional dependencies)
+
+Each Docker image is based on [jupyter/scipy-notebook](https://hub.docker.com/r/jupyter/scipy-notebook)
+and comes with Jupyter environment, vectorbt, and other scientific packages installed.
+
 ## How it works?
 
 vectorbt was implemented to address common performance shortcomings of backtesting libraries.
@@ -54,7 +98,7 @@ Moreover, pandas functions cannot be accessed within user-defined Numba code, si
 compilation on pandas objects. Take for example generating trailing stop orders: to calculate expanding
 maximum for each order, we cannot simply do `df.expanding().max()` from within Numba, but we must write
 and compile our own expanding max function wrapped with `@njit`. That's why vectorbt provides an arsenal
-of Numba-compiled functions for any sort of task.
+of Numba-compiled functions for any sort of tasks.
 
 ## Usability
 
@@ -261,7 +305,7 @@ average, and sell when opposite.
 >>> fast_ma = vbt.MA.run(btc_price, 10, short_name='fast')
 >>> slow_ma = vbt.MA.run(btc_price, 20, short_name='slow')
 
->>> entries = fast_ma.ma_above(slow_ma, crossover=True)
+>>> entries = fast_ma.ma_crossed_above(slow_ma)
 >>> entries
 Date
 2019-01-01 00:00:00+00:00    False
@@ -273,7 +317,7 @@ Date
 2020-01-01 00:00:00+00:00    False
 Freq: D, Length: 366, dtype: bool
 
->>> exits = fast_ma.ma_below(slow_ma, crossover=True)
+>>> exits = fast_ma.ma_crossed_below(slow_ma)
 >>> exits
 Date
 2019-01-01 00:00:00+00:00    False
@@ -301,7 +345,7 @@ average over the entire price series and stores it as a distinct column.
 >>> fast_ma = vbt.MA.run(btc_price, [10, 20], short_name='fast')
 >>> slow_ma = vbt.MA.run(btc_price, [30, 30], short_name='slow')
 
->>> entries = fast_ma.ma_above(slow_ma, crossover=True)
+>>> entries = fast_ma.ma_crossed_above(slow_ma)
 >>> entries
 fast_window                   10     20
 slow_window                   30     30
@@ -316,7 +360,7 @@ Date
 
 [366 rows x 2 columns]
 
->>> exits = fast_ma.ma_below(slow_ma, crossover=True)
+>>> exits = fast_ma.ma_crossed_below(slow_ma)
 >>> exits
 fast_window                   10     20
 slow_window                   30     30
@@ -371,7 +415,7 @@ Date
 >>> fast_ma = vbt.MA.run(comb_price, [10, 20], short_name='fast')
 >>> slow_ma = vbt.MA.run(comb_price, [30, 30], short_name='slow')
 
->>> entries = fast_ma.ma_above(slow_ma, crossover=True)
+>>> entries = fast_ma.ma_crossed_above(slow_ma)
 >>> entries
 fast_window                          10            20
 slow_window                          30            30
@@ -387,7 +431,7 @@ Date
 
 [366 rows x 4 columns]
 
->>> exits = fast_ma.ma_below(slow_ma, crossover=True)
+>>> exits = fast_ma.ma_crossed_below(slow_ma)
 >>> exits
 fast_window                          10            20
 slow_window                          30            30
@@ -416,7 +460,7 @@ Name: total_return, dtype: float64
 >>> mean_return.vbt.barplot(xaxis_title='Symbol', yaxis_title='Mean total return')
 ```
 
-![](/vectorbt/docs/img/index_by_symbol.svg)
+![](/docs/img/index_by_symbol.svg)
 
 Not only strategies and instruments can act as separate features, but also time. If we want to find out
 when our strategy performs best, it's reasonable to test it over multiple time periods. vectorbt allows
@@ -443,8 +487,8 @@ symbol              BTC         ETH           BTC         ETH
 >>> fast_ma = vbt.MA.run(mult_comb_price, [10, 20], short_name='fast')
 >>> slow_ma = vbt.MA.run(mult_comb_price, [30, 30], short_name='slow')
 
->>> entries = fast_ma.ma_above(slow_ma, crossover=True)
->>> exits = fast_ma.ma_below(slow_ma, crossover=True)
+>>> entries = fast_ma.ma_crossed_above(slow_ma)
+>>> exits = fast_ma.ma_crossed_below(slow_ma)
 
 >>> pf = vbt.Portfolio.from_signals(mult_comb_price, entries, exits, freq='1D')
 >>> pf.total_return()
@@ -475,7 +519,7 @@ by any feature, such as window pair, symbol, and time period.
 ...     legend_title_text='Symbol')
 ```
 
-![](/vectorbt/docs/img/index_by_any.svg)
+![](/docs/img/index_by_any.svg)
 
 There is much more to backtesting than simply stacking columns: vectorbt offers functions for
 most parts of a backtesting pipeline, from building indicators and generating signals, to
@@ -505,6 +549,48 @@ Note: you must run the notebook to play with the widgets.
 
 - [Stop Loss, Trailing Stop, or Take Profit? 2 Million Backtests Shed Light](https://polakowo.medium.com/stop-loss-trailing-stop-or-take-profit-2-million-backtests-shed-light-dde23bda40be)
 
+### Getting Help
+
+- If you need supervision or any help with your implementation, [join a private chat](https://www.patreon.com/vectorbt)
+- For questions on Numba and other parts, the best place to go to is [StackOverflow](https://stackoverflow.com/)
+- If you have general questions, start a new [GitHub Discussion](https://github.com/polakowo/vectorbt/discussions)
+  - Alternatively, you can ask on [Gitter](https://gitter.im/vectorbt/community)
+- If you found what appears to be a bug, please [create a new issue](https://github.com/polakowo/vectorbt/issues)
+- For other inquiries, please [contact the author](mailto:olegpolakow@gmail.com)
+
+## Contributing
+
+Pull requests are welcome. For major changes, please open an issue first to discuss what you would like to change.
+
+First, you need to install vectorbt from the repository:
+
+```bash
+pip uninstall vectorbt
+git clone https://github.com/polakowo/vectorbt.git
+cd vectorbt
+pip install -e .
+```
+
+After making changes, make sure you did not break any functionality:
+
+```bash
+pytest
+```
+
+Make sure to update tests as appropriate.
+
+Please note: contribution to this project requires signing a Contributor Licence Agreement (CLA).
+
+## License
+
+This work is [fair-code](http://faircode.io/) distributed under [Apache 2.0 with Commons Clause](https://github.com/polakowo/vectorbt/blob/master/LICENSE.md) license.
+The source code is open and everyone (individuals and organizations) can use it for free.
+However, it is not allowed to sell products and services that are mostly just this software.
+
+If you have any questions about this or want to apply for a license exception, please [contact the author](mailto:olegpolakow@gmail.com).
+
+Installing optional dependencies may be subject to a more restrictive license.
+
 ## Disclaimer
 
 This software is for educational purposes only. Do not risk money which you are afraid to lose.
@@ -515,6 +601,7 @@ __pdoc__ = {}
 
 # Import version
 from vectorbt._version import __version__ as _version
+
 __version__ = _version
 
 # Most important modules
@@ -534,7 +621,7 @@ from vectorbt.labels import *
 from vectorbt.messaging import *
 
 # Import all submodules
-from vectorbt.utils.module import import_submodules
+from vectorbt.utils.module_ import import_submodules
 
 # silence NumbaExperimentalFeatureWarning
 import warnings
