@@ -1,16 +1,23 @@
+# Copyright (c) 2021 Oleg Polakow. All rights reserved.
+# This code is licensed under Apache 2.0 with Commons Clause license (see LICENSE.md for details)
+
 """Functions for reshaping arrays.
 
 Reshape functions transform a pandas object/NumPy array in some way, such as tiling, broadcasting,
 and unstacking."""
+
+import functools
+from collections.abc import Sequence
+
 import numpy as np
 import pandas as pd
 from numba import njit
-from collections.abc import Sequence
+from numpy.lib.stride_tricks import _broadcast_shape
 
 from vectorbt import _typing as tp
+from vectorbt.base import index_fns, array_wrapper
 from vectorbt.utils import checks
 from vectorbt.utils.config import resolve_dict
-from vectorbt.base import index_fns, array_wrapper
 
 
 def to_any_array(arg: tp.ArrayLike, raw: bool = False) -> tp.AnyArray:
@@ -69,6 +76,9 @@ def to_1d(arg: tp.ArrayLike, raw: bool = False) -> tp.AnyArray1d:
     raise ValueError(f"Cannot reshape a {arg.ndim}-dimensional array to 1 dimension")
 
 
+to_1d_array = functools.partial(to_1d, raw=True)
+
+
 def to_2d(arg: tp.ArrayLike, raw: bool = False, expand_axis: int = 1) -> tp.AnyArray2d:
     """Reshape argument to two dimensions. 
 
@@ -89,6 +99,17 @@ def to_2d(arg: tp.ArrayLike, raw: bool = False, expand_axis: int = 1) -> tp.AnyA
     raise ValueError(f"Cannot reshape a {arg.ndim}-dimensional array to 2 dimensions")
 
 
+to_2d_array = functools.partial(to_2d, raw=True)
+
+
+def to_dict(arg: tp.ArrayLike, orient: str = 'dict') -> dict:
+    """Convert object to dict."""
+    arg = to_pd_array(arg)
+    if orient == 'index_series':
+        return {arg.index[i]: arg.iloc[i] for i in range(len(arg.index))}
+    return arg.to_dict(orient)
+
+
 def repeat(arg: tp.ArrayLike, n: int, axis: int = 1, raw: bool = False) -> tp.AnyArray:
     """Repeat each element in `arg` `n` times along the specified axis."""
     arg = to_any_array(arg, raw=raw)
@@ -153,7 +174,7 @@ def broadcast_index(args: tp.Sequence[tp.AnyArray],
             * 'stack' - stack different indexes/columns using `vectorbt.base.index_fns.stack_indexes`
             * 'strict' - ensure that all pandas objects have the same index/columns
             * 'reset' - reset any index/columns (they become a simple range)
-            * integer - use the index/columns of the i-nth object in `args`
+            * integer - use the index/columns of the i-th object in `args`
             * everything else will be converted to `pd.Index`
 
         axis (int): Set to 0 for index and 1 for columns.
@@ -296,16 +317,6 @@ def wrap_broadcasted(old_arg: tp.AnyArray,
     return new_arg
 
 
-def broadcast_shape(*args) -> tp.Shape:
-    """Returns the shape of the arrays that would result from broadcasting the
-    supplied arrays against each other."""
-    b = np.broadcast(*args[:32])
-    for pos in range(32, len(args), 31):
-        b = broadcast_to(0, b.shape)
-        b = np.broadcast(b, *args[pos:(pos + 31)])
-    return b.shape
-
-
 BCRT = tp.Union[
     tp.MaybeTuple[tp.AnyArray],
     tp.Tuple[tp.MaybeTuple[tp.AnyArray], tp.Shape, tp.Optional[tp.Index], tp.Optional[tp.Index]]
@@ -507,7 +518,7 @@ def broadcast(*args: tp.ArrayLike,
     if to_shape is not None:
         if isinstance(to_shape, int):
             to_shape = (to_shape,)
-        checks.assert_type(to_shape, tuple)
+        checks.assert_instance_of(to_shape, tuple)
         if len(to_shape) > 1:
             is_2d = True
 
@@ -551,7 +562,7 @@ def broadcast(*args: tp.ArrayLike,
 
     # Get final shape
     if to_shape is None:
-        to_shape = broadcast_shape(*arr_args_2d)
+        to_shape = _broadcast_shape(*map(np.asarray, arr_args_2d))
 
     # Perform broadcasting
     new_args = []
@@ -710,7 +721,7 @@ def get_multiindex_series(arg: tp.SeriesFrame) -> tp.Series:
     """Get Series with a multi-index.
 
     If DataFrame has been passed, should at maximum have one row or column."""
-    checks.assert_type(arg, (pd.Series, pd.DataFrame))
+    checks.assert_instance_of(arg, (pd.Series, pd.DataFrame))
     if checks.is_frame(arg):
         if arg.shape[0] == 1:
             arg = arg.iloc[0, :]
@@ -718,7 +729,7 @@ def get_multiindex_series(arg: tp.SeriesFrame) -> tp.Series:
             arg = arg.iloc[:, 0]
         else:
             raise ValueError("Supported are either Series or DataFrame with one column/row")
-    checks.assert_type(arg.index, pd.MultiIndex)
+    checks.assert_instance_of(arg.index, pd.MultiIndex)
     return arg
 
 
@@ -804,11 +815,11 @@ def make_symmetric(arg: tp.SeriesFrame, sort: bool = True) -> tp.Frame:
     d  2.0  4.0  NaN  NaN
     ```
     """
-    checks.assert_type(arg, (pd.Series, pd.DataFrame))
+    checks.assert_instance_of(arg, (pd.Series, pd.DataFrame))
     df: tp.Frame = to_2d(arg)
     if isinstance(df.index, pd.MultiIndex) or isinstance(df.columns, pd.MultiIndex):
-        checks.assert_type(df.index, pd.MultiIndex)
-        checks.assert_type(df.columns, pd.MultiIndex)
+        checks.assert_instance_of(df.index, pd.MultiIndex)
+        checks.assert_instance_of(df.columns, pd.MultiIndex)
         checks.assert_array_equal(df.index.nlevels, df.columns.nlevels)
         names1, names2 = tuple(df.index.names), tuple(df.columns.names)
     else:
@@ -825,24 +836,31 @@ def make_symmetric(arg: tp.SeriesFrame, sort: bool = True) -> tp.Frame:
         idx_vals = np.unique(np.concatenate((df.index, df.columns))).tolist()
     else:
         idx_vals = list(dict.fromkeys(np.concatenate((df.index, df.columns))))
-    df = df.copy()
+    df_index = df.index.copy()
+    df_columns = df.columns.copy()
     if isinstance(df.index, pd.MultiIndex):
         unique_index = pd.MultiIndex.from_tuples(idx_vals, names=new_name)
-        df.index.names = new_name
-        df.columns.names = new_name
+        df_index.names = new_name
+        df_columns.names = new_name
     else:
         unique_index = pd.Index(idx_vals, name=new_name)
-        df.index.name = new_name
-        df.columns.name = new_name
-    df_out = pd.DataFrame(index=unique_index, columns=unique_index)
+        df_index.name = new_name
+        df_columns.name = new_name
+    df = df.copy(deep=False)
+    df.index = df_index
+    df.columns = df_columns
+    df_out_dtype = np.promote_types(df.values.dtype, np.min_scalar_type(np.nan))
+    df_out = pd.DataFrame(index=unique_index, columns=unique_index, dtype=df_out_dtype)
     df_out.loc[:, :] = df
     df_out[df_out.isnull()] = df.transpose()
     return df_out
 
 
-def unstack_to_df(arg: tp.SeriesFrame, index_levels: tp.Optional[tp.MaybeLevelSequence] = None,
+def unstack_to_df(arg: tp.SeriesFrame,
+                  index_levels: tp.Optional[tp.MaybeLevelSequence] = None,
                   column_levels: tp.Optional[tp.MaybeLevelSequence] = None,
-                  symmetric: bool = False, sort: bool = True) -> tp.Frame:
+                  symmetric: bool = False,
+                  sort: bool = True) -> tp.Frame:
     """Reshape `arg` based on its multi-index into a DataFrame.
 
     Use `index_levels` to specify what index levels will form new index, and `column_levels` 
@@ -895,12 +913,12 @@ def unstack_to_df(arg: tp.SeriesFrame, index_levels: tp.Optional[tp.MaybeLevelSe
 
 
 @njit(cache=True)
-def flex_choose_i_and_col_nb(a: tp.Array, flex_2d: bool) -> tp.Tuple[int, int]:
+def flex_choose_i_and_col_nb(a: tp.Array, flex_2d: bool = True) -> tp.Tuple[int, int]:
     """Choose selection index and column based on the array's shape.
 
-    Instead of expensive broadcasting, keep original shape and do indexing in a smart way.
+    Instead of expensive broadcasting, keep the original shape and do indexing in a smart way.
     A nice feature of this is that it has almost no memory footprint and can broadcast in
-    any direction indefinitely.
+    any direction infinitely.
 
     Call it once before using `flex_select_nb`.
 
@@ -928,7 +946,7 @@ def flex_choose_i_and_col_nb(a: tp.Array, flex_2d: bool) -> tp.Tuple[int, int]:
 
 
 @njit(cache=True)
-def flex_select_nb(i: int, col: int, a: tp.Array, flex_i: int, flex_col: int, flex_2d: bool) -> tp.Any:
+def flex_select_nb(a: tp.Array, i: int, col: int, flex_i: int, flex_col: int, flex_2d: bool = True) -> tp.Any:
     """Select element of `a` as if it has been broadcast."""
     if flex_i == -1:
         flex_i = i
@@ -944,10 +962,7 @@ def flex_select_nb(i: int, col: int, a: tp.Array, flex_i: int, flex_col: int, fl
 
 
 @njit(cache=True)
-def flex_select_auto_nb(i: int, col: int, a: tp.Array, flex_2d: bool) -> tp.Any:
-    """Combines `flex_choose_i_and_col_nb` and `flex_select_nb`.
-
-    !!! note
-        Slower since it must call `flex_choose_i_and_col_nb` each time."""
+def flex_select_auto_nb(a: tp.Array, i: int, col: int, flex_2d: bool = True) -> tp.Any:
+    """Combines `flex_choose_i_and_col_nb` and `flex_select_nb`."""
     flex_i, flex_col = flex_choose_i_and_col_nb(a, flex_2d)
-    return flex_select_nb(i, col, a, flex_i, flex_col, flex_2d)
+    return flex_select_nb(a, i, col, flex_i, flex_col, flex_2d)
