@@ -1,21 +1,25 @@
+# Copyright (c) 2021 Oleg Polakow. All rights reserved.
+# This code is licensed under Apache 2.0 with Commons Clause license (see LICENSE.md for details)
+
 """Custom data classes that subclass `vectorbt.data.base.Data`."""
 
-import numpy as np
-import pandas as pd
-from tqdm import tqdm
 import time
 import warnings
 from functools import wraps
 
+import numpy as np
+import pandas as pd
+from tqdm.auto import tqdm
+
 from vectorbt import _typing as tp
-from vectorbt.utils.datetime import (
+from vectorbt.data.base import Data
+from vectorbt.utils.config import merge_dicts, get_func_kwargs
+from vectorbt.utils.datetime_ import (
     get_utc_tz,
     get_local_tz,
     to_tzaware_datetime,
     datetime_to_ms
 )
-from vectorbt.utils.config import merge_dicts, get_func_kwargs
-from vectorbt.data.base import Data
 
 try:
     from binance.client import Client as ClientT
@@ -194,7 +198,7 @@ class YFData(Data):
     ...     interval='1m'
     ... )
     >>> yf_data.get())
-                                     Open        High         Low       Close  \
+                                     Open        High         Low       Close  \\
     Datetime
     2021-04-12 13:30:00+00:00  685.080017  685.679993  684.765015  685.679993
     2021-04-12 13:31:00+00:00  684.625000  686.500000  684.010010  685.500000
@@ -212,7 +216,7 @@ class YFData(Data):
 
     >>> yf_data = yf_data.update(end='2021-04-12 09:40:00 -0400')
     >>> yf_data.get()
-                                     Open        High         Low       Close  \
+                                     Open        High         Low       Close  \\
     Datetime
     2021-04-12 13:30:00+00:00  685.080017  685.679993  684.765015  685.679993
     2021-04-12 13:31:00+00:00  684.625000  686.500000  684.010010  685.500000
@@ -254,10 +258,10 @@ class YFData(Data):
             period (str): Period.
             start (any): Start datetime.
 
-                See `vectorbt.utils.datetime.to_tzaware_datetime`.
+                See `vectorbt.utils.datetime_.to_tzaware_datetime`.
             end (any): End datetime.
 
-                See `vectorbt.utils.datetime.to_tzaware_datetime`.
+                See `vectorbt.utils.datetime_.to_tzaware_datetime`.
             **kwargs: Keyword arguments passed to `yfinance.base.TickerBase.history`.
         """
         import yfinance as yf
@@ -301,7 +305,7 @@ class BinanceData(Data):
     ... )
     >>> binance_data.get()
     2021-05-02 14:47:20.478000+00:00 - 2021-05-02 16:47:00+00:00: : 1it [00:00,  3.42it/s]
-                                   Open      High       Low     Close     Volume  \
+                                   Open      High       Low     Close     Volume  \\
     Open time
     2021-05-02 14:48:00+00:00  56867.44  56913.57  56857.40  56913.56  28.709976
     2021-05-02 14:49:00+00:00  56913.56  56913.57  56845.94  56888.00  19.734841
@@ -311,7 +315,7 @@ class BinanceData(Data):
     2021-05-02 16:46:00+00:00  56644.02  56663.43  56605.17  56605.18  27.573654
     2021-05-02 16:47:00+00:00  56605.18  56657.55  56605.17  56627.12   7.719933
 
-                                                    Close time  Quote volume  \
+                                                    Close time  Quote volume  \\
     Open time
     2021-05-02 14:48:00+00:00 2021-05-02 14:48:59.999000+00:00  1.633534e+06
     2021-05-02 14:49:00+00:00 2021-05-02 14:49:59.999000+00:00  1.122519e+06
@@ -321,7 +325,7 @@ class BinanceData(Data):
     2021-05-02 16:46:00+00:00 2021-05-02 16:46:59.999000+00:00  1.561548e+06
     2021-05-02 16:47:00+00:00 2021-05-02 16:47:59.999000+00:00  4.371848e+05
 
-                               Number of trades  Taker base volume  \
+                               Number of trades  Taker base volume  \\
     Open time
     2021-05-02 14:48:00+00:00               991          13.771152
     2021-05-02 14:49:00+00:00               816           5.981942
@@ -348,7 +352,7 @@ class BinanceData(Data):
 
     >>> binance_data = binance_data.update()
     >>> binance_data.get()
-                                   Open      High       Low     Close     Volume  \
+                                   Open      High       Low     Close     Volume  \\
     Open time
     2021-05-02 14:48:00+00:00  56867.44  56913.57  56857.40  56913.56  28.709976
     2021-05-02 14:49:00+00:00  56913.56  56913.57  56845.94  56888.00  19.734841
@@ -358,7 +362,7 @@ class BinanceData(Data):
     2021-05-02 16:47:00+00:00  56605.18  56657.55  56605.17  56625.76  14.615437
     2021-05-02 16:48:00+00:00  56625.75  56643.60  56614.32  56623.01   5.895843
 
-                                                    Close time  Quote volume  \
+                                                    Close time  Quote volume  \\
     Open time
     2021-05-02 14:48:00+00:00 2021-05-02 14:48:59.999000+00:00  1.633534e+06
     2021-05-02 14:49:00+00:00 2021-05-02 14:49:59.999000+00:00  1.122519e+06
@@ -368,7 +372,7 @@ class BinanceData(Data):
     2021-05-02 16:47:00+00:00 2021-05-02 16:47:59.999000+00:00  8.276017e+05
     2021-05-02 16:48:00+00:00 2021-05-02 16:48:59.999000+00:00  3.338702e+05
 
-                               Number of trades  Taker base volume  \
+                               Number of trades  Taker base volume  \\
     Open time
     2021-05-02 14:48:00+00:00               991          13.771152
     2021-05-02 14:49:00+00:00               816           5.981942
@@ -419,7 +423,8 @@ class BinanceData(Data):
                         end: tp.DatetimeLike = 'now UTC',
                         delay: tp.Optional[float] = 500,
                         limit: int = 500,
-                        show_progress: bool = True) -> tp.Frame:
+                        show_progress: bool = True,
+                        tqdm_kwargs: tp.KwargsLike = None) -> tp.Frame:
         """Download the symbol.
 
         Args:
@@ -430,19 +435,22 @@ class BinanceData(Data):
                 See `binance.enums`.
             start (any): Start datetime.
 
-                See `vectorbt.utils.datetime.to_tzaware_datetime`.
+                See `vectorbt.utils.datetime_.to_tzaware_datetime`.
             end (any): End datetime.
 
-                See `vectorbt.utils.datetime.to_tzaware_datetime`.
+                See `vectorbt.utils.datetime_.to_tzaware_datetime`.
             delay (float): Time to sleep after each request (in milliseconds).
             limit (int): The maximum number of returned items.
             show_progress (bool): Whether to show the progress bar.
+            tqdm_kwargs (dict): Keyword arguments passed to `tqdm`.
 
         For defaults, see `data.binance` in `vectorbt._settings.settings`.
         """
         if client is None:
             raise ValueError("client must be provided")
 
+        if tqdm_kwargs is None:
+            tqdm_kwargs = {}
         # Establish the timestamps
         start_ts = datetime_to_ms(to_tzaware_datetime(start, tz=get_utc_tz()))
         try:
@@ -464,7 +472,7 @@ class BinanceData(Data):
 
         # Iteratively collect the data
         data: tp.List[list] = []
-        with tqdm(disable=not show_progress) as pbar:
+        with tqdm(disable=not show_progress, **tqdm_kwargs) as pbar:
             pbar.set_description(_ts_to_str(start_ts))
             while True:
                 # Fetch the klines for the next interval
@@ -595,7 +603,8 @@ class CCXTData(Data):
                         limit: tp.Optional[int] = 500,
                         retries: int = 3,
                         show_progress: bool = True,
-                        params: tp.Optional[dict] = None) -> tp.Frame:
+                        params: tp.Optional[dict] = None,
+                        tqdm_kwargs: tp.KwargsLike = None) -> tp.Frame:
         """Download the symbol.
 
         Args:
@@ -608,10 +617,10 @@ class CCXTData(Data):
             timeframe (str): Timeframe supported by the exchange.
             start (any): Start datetime.
 
-                See `vectorbt.utils.datetime.to_tzaware_datetime`.
+                See `vectorbt.utils.datetime_.to_tzaware_datetime`.
             end (any): End datetime.
 
-                See `vectorbt.utils.datetime.to_tzaware_datetime`.
+                See `vectorbt.utils.datetime_.to_tzaware_datetime`.
             delay (float): Time to sleep after each request (in milliseconds).
 
                 !!! note
@@ -619,6 +628,7 @@ class CCXTData(Data):
             limit (int): The maximum number of returned items.
             retries (int): The number of retries on failure to fetch data.
             show_progress (bool): Whether to show the progress bar.
+            tqdm_kwargs (dict): Keyword arguments passed to `tqdm`.
             params (dict): Exchange-specific key-value parameters.
 
         For defaults, see `data.ccxt` in `vectorbt._settings.settings`.
@@ -629,6 +639,8 @@ class CCXTData(Data):
 
         if config is None:
             config = {}
+        if tqdm_kwargs is None:
+            tqdm_kwargs = {}
         if params is None:
             params = {}
         if isinstance(exchange, str):
@@ -694,7 +706,7 @@ class CCXTData(Data):
 
         # Iteratively collect the data
         data: tp.List[list] = []
-        with tqdm(disable=not show_progress) as pbar:
+        with tqdm(disable=not show_progress, **tqdm_kwargs) as pbar:
             pbar.set_description(_ts_to_str(start_ts))
             while True:
                 # Fetch the klines for the next interval
@@ -745,3 +757,170 @@ class CCXTData(Data):
         download_kwargs['show_progress'] = False
         kwargs = merge_dicts(download_kwargs, kwargs)
         return self.download_symbol(symbol, **kwargs)
+
+class AlpacaData(Data):
+    """`Data` for data coming from `alpaca-trade-api`. 
+    Sign up for Alpaca API keys here: https://app.alpaca.markets/signup
+    
+    ## Example
+
+    Fetch the 1-minute data of the last 2 hours, wait 1 minute, and update:
+
+    ```python-repl
+    >>> import vectorbt as vbt
+
+    >>> alpaca_data = vbt.AlpacaData.download(
+    ...     "AAPL",
+    ...     start='2 hours ago UTC',
+    ...     end='15 minutes ago UTC',
+    ...     interval='1m'
+    ... )
+    >>> alpaca_data.get()
+                                Open      High       Low     Close      Volume
+    timestamp                                                                
+    2021-12-27 14:04:00+00:00  177.0500  177.0500  177.0500  177.0500    1967
+    2021-12-27 14:05:00+00:00  177.0500  177.0500  177.0300  177.0500    3218
+    2021-12-27 14:06:00+00:00  177.0400  177.0400  177.0400  177.0400     873
+    2021-12-27 14:07:00+00:00  177.0399  177.0400  177.0300  177.0400    1100
+    2021-12-27 14:08:00+00:00  177.0400  177.0400  176.9700  176.9900   19943
+    ...                             ...       ...       ...       ...     ...
+    2021-12-27 15:44:00+00:00  178.0000  178.0079  177.9311  177.9710  106395
+    2021-12-27 15:45:00+00:00  177.9700  178.0564  177.9430  177.9600  153325
+    2021-12-27 15:46:00+00:00  177.9500  178.0000  177.8289  177.8850  162778
+    2021-12-27 15:47:00+00:00  177.8810  177.9600  177.8400  177.9515  123284
+    2021-12-27 15:48:00+00:00  177.9600  178.0500  177.9600  178.0100  159700
+
+    [105 rows x 5 columns]
+
+    >>> import time
+    >>> time.sleep(60)
+
+    >>> alpaca_data = alpaca_data.update()
+    >>> alpaca_data.get()
+                                Open      High       Low     Close      Volume
+    timestamp                                                                
+    2021-12-27 14:04:00+00:00  177.0500  177.0500  177.0500  177.0500    1967
+    2021-12-27 14:05:00+00:00  177.0500  177.0500  177.0300  177.0500    3218
+    2021-12-27 14:06:00+00:00  177.0400  177.0400  177.0400  177.0400     873
+    2021-12-27 14:07:00+00:00  177.0399  177.0400  177.0300  177.0400    1100
+    2021-12-27 14:08:00+00:00  177.0400  177.0400  176.9700  176.9900   19943
+    ...                             ...       ...       ...       ...     ...
+    2021-12-27 15:45:00+00:00  177.9700  178.0564  177.9430  177.9600  153325
+    2021-12-27 15:46:00+00:00  177.9500  178.0000  177.8289  177.8850  162778
+    2021-12-27 15:47:00+00:00  177.8810  177.9600  177.8400  177.9515  123284
+    2021-12-27 15:48:00+00:00  177.9600  178.0500  177.9600  178.0100  159700
+    2021-12-27 15:49:00+00:00  178.0100  178.0700  177.9700  178.0650  185037
+
+    [106 rows x 5 columns]
+    ```"""
+
+    @classmethod
+    def download_symbol(cls,
+                        symbol: str,
+                        timeframe: str = '1d',
+                        start: tp.DatetimeLike = 0,
+                        end: tp.DatetimeLike = 'now UTC',
+                        adjustment: tp.Optional[str] = 'all',
+                        limit: int = 500,
+                        exchange: tp.Optional[str] = 'CBSE',
+                        **kwargs) -> tp.Frame:
+        """Download the symbol.
+
+        Args:
+            symbol (str): Symbol.
+            timeframe (str): Timeframe of data. Must be integer multiple of 'm' (minute), 'h' (hour) or 'd' (day). i.e. '15m' 
+                Note: Data from the latest 15 minutes is not available with a free data plan.
+                See: https://alpaca.markets/data
+
+            start (any): Start datetime.
+
+                See `vectorbt.utils.datetime_.to_tzaware_datetime`.
+            end (any): End datetime.
+
+                See `vectorbt.utils.datetime_.to_tzaware_datetime`.
+            adjustment (str): Specifies the corporate action adjustment for the stocks. 
+                Enum: `raw`, `split`, `dividend` or `all`.
+            limit (int): The maximum number of returned items.
+            exchange (str): For crypto symbols. Which exchange you wish to retrieve data from.
+                Enum: `FTX`, `ERSX`, `CBSE`
+
+        For defaults, see `data.alpaca` in `vectorbt._settings.settings`.
+        """
+        from vectorbt._settings import settings
+        from alpaca_trade_api.rest import TimeFrameUnit, TimeFrame, REST
+        
+        alpaca_cfg = settings['data']['alpaca']
+        
+        client_kwargs = dict()
+        for k in get_func_kwargs(REST):
+            if k in kwargs:
+                client_kwargs[k] = kwargs.pop(k)
+
+        client_kwargs = merge_dicts(alpaca_cfg, client_kwargs)
+        
+        client = REST(**client_kwargs)
+
+        _timeframe_units = {'d': TimeFrameUnit.Day, 'h': TimeFrameUnit.Hour, 'm': TimeFrameUnit.Minute}
+
+        if len(timeframe) < 2:
+            raise ValueError("invalid timeframe")
+            
+        amount_str = timeframe[:-1]
+        unit_str = timeframe[-1]
+
+        if not amount_str.isnumeric() or unit_str not in _timeframe_units:
+            raise ValueError("invalid timeframe")
+        
+        amount = int(amount_str)
+        unit = _timeframe_units[unit_str]
+
+        _timeframe = TimeFrame(amount, unit)
+
+        start_ts = to_tzaware_datetime(start, tz=get_utc_tz()).isoformat()
+        end_ts = to_tzaware_datetime(end, tz=get_utc_tz()).isoformat()
+
+        
+        def _is_crypto_symbol(symbol):
+            return len(symbol) == 6 and "USD" in symbol
+
+        if _is_crypto_symbol(symbol):
+            df = client.get_crypto_bars(symbol=symbol,
+                                    timeframe=_timeframe,
+                                    start=start_ts,
+                                    end=end_ts,
+                                    limit=limit,
+                                    exchanges=exchange).df
+        else:
+            df = client.get_bars(symbol=symbol, 
+                                timeframe=_timeframe,
+                                start=start_ts,
+                                end=end_ts,
+                                adjustment=adjustment, 
+                                limit=limit).df
+        
+        # filter for OHLCV
+        # remove extra columns
+        df.drop(['trade_count', 'vwap'], axis=1, errors='ignore', inplace=True)
+
+        # capitalize
+        df.rename(columns={'open': 'Open', 'high': 'High', 'low': 'Low', 'close': 'Close', 'volume': 'Volume', 'exchange':'Exchange'}, inplace=True)
+
+        df['Open'] = df['Open'].astype(float)
+        df['High'] = df['High'].astype(float)
+        df['Low'] = df['Low'].astype(float)
+        df['Close'] = df['Close'].astype(float)
+        df['Volume'] = df['Volume'].astype(float)
+
+        return df
+
+
+
+    def update_symbol(self, symbol: str, **kwargs) -> tp.Frame:
+        """Update the symbol.
+
+        `**kwargs` will override keyword arguments passed to `AlpacaData.download_symbol`."""
+        download_kwargs = self.select_symbol_kwargs(symbol, self.download_kwargs)
+        download_kwargs['start'] = self.data[symbol].index[-1]
+        download_kwargs['show_progress'] = False
+        kwargs = merge_dicts(download_kwargs, kwargs)
+        return self.download_symbol(symbol, **kwargs)
\ No newline at end of file
