@@ -1,3 +1,6 @@
+# Copyright (c) 2021 Oleg Polakow. All rights reserved.
+# This code is licensed under Apache 2.0 with Commons Clause license (see LICENSE.md for details)
+
 """Indicators built with `vectorbt.indicators.factory.IndicatorFactory`.
 
 You can access all the indicators either by `vbt.*` or `vbt.indicators.*`.
@@ -30,7 +33,7 @@ Run for the examples below:
 >>> cols = ['Open', 'High', 'Low', 'Close', 'Volume']
 >>> ohlcv = vbt.YFData.download("BTC-USD", start=start, end=end).get(cols)
 >>> ohlcv
-                                   Open          High          Low  \
+                                   Open          High          Low  \\
 Date
 2019-03-01 00:00:00+00:00   3853.757080   3907.795410  3851.692383
 2019-03-02 00:00:00+00:00   3855.318115   3874.607422  3832.127930
@@ -54,18 +57,18 @@ Date
 
 >>> ohlcv.vbt.ohlcv.plot()
 ```
-![](/vectorbt/docs/img/basic_price.svg)"""
+![](/docs/img/basic_price.svg)"""
 
 import numpy as np
 import plotly.graph_objects as go
 
 from vectorbt import _typing as tp
+from vectorbt.generic import nb as generic_nb
+from vectorbt.indicators import nb
+from vectorbt.indicators.factory import IndicatorFactory
+from vectorbt.utils.colors import adjust_opacity
 from vectorbt.utils.config import merge_dicts
 from vectorbt.utils.figure import make_figure
-from vectorbt.utils.colors import adjust_opacity
-from vectorbt.generic import nb as generic_nb
-from vectorbt.indicators.factory import IndicatorFactory
-from vectorbt.indicators import nb
 
 # ############# MA ############# #
 
@@ -119,7 +122,7 @@ class _MA(MA):
         >>> vbt.MA.run(ohlcv['Close'], 10).plot()
         ```
 
-        ![](/vectorbt/docs/img/MA.svg)
+        ![](/docs/img/MA.svg)
         """
         from vectorbt._settings import settings
         plotting_cfg = settings['plotting']
@@ -205,7 +208,7 @@ class _MSTD(MSTD):
         >>> vbt.MSTD.run(ohlcv['Close'], 10).plot()
         ```
 
-        ![](/vectorbt/docs/img/MSTD.svg)
+        ![](/docs/img/MSTD.svg)
         """
         self_col = self.select_one(column=column)
 
@@ -296,7 +299,7 @@ class _BBANDS(BBANDS):
         >>> vbt.BBANDS.run(ohlcv['Close']).plot()
         ```
 
-        ![](/vectorbt/docs/img/BBANDS.svg)
+        ![](/docs/img/BBANDS.svg)
         """
         from vectorbt._settings import settings
         plotting_cfg = settings['plotting']
@@ -392,7 +395,7 @@ class _RSI(RSI):
              levels: tp.Tuple[float, float] = (30, 70),
              rsi_trace_kwargs: tp.KwargsLike = None,
              add_trace_kwargs: tp.KwargsLike = None,
-             xref: str = 'x', 
+             xref: str = 'x',
              yref: str = 'y',
              fig: tp.Optional[tp.BaseFigure] = None,
              **layout_kwargs) -> tp.BaseFigure:  # pragma: no cover
@@ -414,7 +417,7 @@ class _RSI(RSI):
         >>> vbt.RSI.run(ohlcv['Close']).plot()
         ```
 
-        ![](/vectorbt/docs/img/RSI.svg)
+        ![](/docs/img/RSI.svg)
         """
         self_col = self.select_one(column=column)
 
@@ -517,7 +520,7 @@ class _STOCH(STOCH):
         >>> vbt.STOCH.run(ohlcv['High'], ohlcv['Low'], ohlcv['Close']).plot()
         ```
 
-        ![](/vectorbt/docs/img/STOCH.svg)
+        ![](/docs/img/STOCH.svg)
         """
         self_col = self.select_one(column=column)
 
@@ -630,7 +633,7 @@ class _MACD(MACD):
         >>> vbt.MACD.run(ohlcv['Close']).plot()
         ```
 
-        ![](/vectorbt/docs/img/MACD.svg)"""
+        ![](/docs/img/MACD.svg)"""
         self_col = self.select_one(column=column)
 
         if fig is None:
@@ -741,7 +744,7 @@ class _ATR(ATR):
         >>> vbt.ATR.run(ohlcv['High'], ohlcv['Low'], ohlcv['Close'], 10).plot()
         ```
 
-        ![](/vectorbt/docs/img/ATR.svg)
+        ![](/docs/img/ATR.svg)
         """
         self_col = self.select_one(column=column)
 
@@ -814,7 +817,7 @@ class _OBV(OBV):
         >>> vbt.OBV.run(ohlcv['Close'], ohlcv['Volume']).plot()
         ```
 
-        ![](/vectorbt/docs/img/OBV.svg)
+        ![](/docs/img/OBV.svg)
         """
         self_col = self.select_one(column=column)
 
