@@ -1,8 +1,13 @@
+# Copyright (c) 2021 Oleg Polakow. All rights reserved.
+# This code is licensed under Apache 2.0 with Commons Clause license (see LICENSE.md for details)
+
 """Messaging using `python-telegram-bot`."""
 
 import logging
 from functools import wraps
+
 from telegram import Update
+from telegram.error import Unauthorized, ChatMigrated
 from telegram.ext import (
     Handler,
     CallbackContext,
@@ -15,11 +20,10 @@ from telegram.ext import (
     Defaults
 )
 from telegram.utils.helpers import effective_message_type
-from telegram.error import Unauthorized, ChatMigrated
 
 from vectorbt import _typing as tp
 from vectorbt.utils.config import merge_dicts, get_func_kwargs, Configured
-from vectorbt.utils.requests import text_to_giphy_url
+from vectorbt.utils.requests_ import text_to_giphy_url
 
 logger = logging.getLogger(__name__)
 
@@ -149,7 +153,7 @@ class TelegramBot(Configured):
         self.giphy_kwargs = giphy_kwargs
         default_kwargs = dict()
         passed_kwargs = dict()
-        for k in get_func_kwargs(Updater):
+        for k in get_func_kwargs(Updater.__init__):
             if k in telegram_cfg:
                 default_kwargs[k] = telegram_cfg[k]
             if k in kwargs:
