@@ -1,16 +1,21 @@
+# Copyright (c) 2021 Oleg Polakow. All rights reserved.
+# This code is licensed under Apache 2.0 with Commons Clause license (see LICENSE.md for details)
+
 """Modules for working with portfolios."""
 
-from vectorbt.portfolio.enums import *
 from vectorbt.portfolio.base import Portfolio
-from vectorbt.portfolio.orders import Orders
+from vectorbt.portfolio.enums import *
 from vectorbt.portfolio.logs import Logs
-from vectorbt.portfolio.trades import Trades, Positions
+from vectorbt.portfolio.orders import Orders
+from vectorbt.portfolio.trades import Trades, EntryTrades, ExitTrades, Positions
 
 __all__ = [
     'Portfolio',
     'Orders',
     'Logs',
     'Trades',
+    'EntryTrades',
+    'ExitTrades',
     'Positions'
 ]
 
