@@ -1,12 +1,15 @@
+# Copyright (c) 2021 Oleg Polakow. All rights reserved.
+# This code is licensed under Apache 2.0 with Commons Clause license (see LICENSE.md for details)
+
 """Modules with utilities that are used throughout vectorbt."""
 
 from vectorbt.utils.config import atomic_dict, merge_dicts, Config, Configured, AtomicConfig
-from vectorbt.utils.template import Sub, Rep, RepEval, RepFunc, deep_substitute
 from vectorbt.utils.decorators import CacheCondition, cached_property, cached_method
 from vectorbt.utils.figure import Figure, FigureWidget, make_figure, make_subplots
-from vectorbt.utils.random import set_seed
-from vectorbt.utils.image import save_animation
-from vectorbt.utils.schedule import AsyncJob, AsyncScheduler, CancelledError, ScheduleManager
+from vectorbt.utils.image_ import save_animation
+from vectorbt.utils.random_ import set_seed
+from vectorbt.utils.schedule_ import AsyncJob, AsyncScheduler, CancelledError, ScheduleManager
+from vectorbt.utils.template import Sub, Rep, RepEval, RepFunc, deep_substitute
 
 __all__ = [
     'atomic_dict',
